using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.IO;
using System.Diagnostics;

namespace Proyecto0
{

    /// <sumary>
    ///  Formulario principal que mantiene el control del juego
    /// </sumary>
    public class DamasForm : Form
    {

        /// <summary>
        /// Variable del diseador requerida.
        /// </summary>
        private System.ComponentModel.IContainer components = null;
        private System.Windows.Forms.MainMenu MenuPrincipal;
        private System.Windows.Forms.MenuItem menuItem1;
        private System.Windows.Forms.MenuItem menuItem2;
        private MenuItem menuItem4;
        private MenuItem menuItem5;
        private MenuItem menuItem6;
        private MenuItem menuItem7;
        private MenuItem menuItem8;
        private MenuItem menuItem9;
        private MenuItem menuItem10;
        private MenuItem menuItem11;
        private MenuItem menuItem12;
        private MenuItem menuItem13;
        private PictureBox pictureBox1;
        private System.Windows.Forms.MenuItem menuItem3;


        // Partida que lleva el control del juego
        private Partida partida;
        // Instancia nica de la clase
        private static DamasForm Instance = null;
        // Imagen de fondo de la pantalla de juego
        private Image imagenFondo;
        // Rectngulo en el que se encuadra la imagen de fondo
        private Rectangle rectFondo;

        /// <summary>
        /// Limpiar los recursos que se estn utilizando.
        /// </summary>
        /// <param name="disposing">true si los recursos administrados se deben eliminar; false en caso contrario, false.</param>
        protected override void Dispose(bool disposing)
        {
            if (disposing && (components != null))
            {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

        /// <sumary> 
        ///  Crea el form principal y da el control de juego a la clase Partida
        /// </sumary>
        private DamasForm()
        {
            InitializeComponent();

            // Da el control de la pantalla a la partida
            partida = new Partida(this);
            partida.Location = new Point(18, 19);
            //partida.Size = ClientSize;
            partida.Size = new Size(200, 200);
            Controls.Add(partida);
        }

        /// <sumary> 
        ///  Obtiene una nica instancia de la clase (Patrn Singleton)
        /// </sumary>
        public static DamasForm getInstance()
        {
            if (Instance == null)
            {
                Instance = new DamasForm();
            }
            return Instance;
        }

        /// <sumary> 
        ///  Cambia el pictureBox que indica el turno del jugador
        /// </sumary> 
        public void cambiaTurno(Image turno) 
        {
            this.pictureBox1.Image = turno;
            Invalidate();
            Update();
        }

        /// <sumary> 
        ///  Cambia la dificultad (1-3) del juego en modo 1 Jugador
        /// </sumary> 
        public void cambiaDificultad(int d)
        {
            partida.dificultad = d;
            if (d == 2)
            // Dificultad baja
            {
                menuItem9.Checked = true;
                menuItem10.Checked = false;
                menuItem11.Checked = false;
            }
            // Dificultad media
            else if (d == 4)
            {
                menuItem9.Checked = false;
                menuItem10.Checked = true;
                menuItem11.Checked = false;
            }
            // Dificultad alta
            else if (d == 5)
            {
                menuItem9.Checked = false;
                menuItem10.Checked = false;
                menuItem11.Checked = true;
            }
        }


        /// <sumary> 
        ///  Cambia el nmero de jugadores (1-2) de la partida
        /// </sumary> 
        public void cambiaNumJugadores(int n)
        {
            // Jugador vs CPU
            if (n == 1)
            {
                partida.juegaCPU = true;
                menuItem6.Checked = true;
                menuItem5.Checked = false;
            }
            // 2 jugadores
            else if (n == 2)
            {
                partida.juegaCPU = false;
                menuItem5.Checked = true;
                menuItem6.Checked = false;
            }
        }

        /// <sumary> 
        ///  Muestra el archivo de ayuda de la aplicacin
        /// </sumary> 
        public void cargarAyuda()
        {
            String ruta;

            try
            {
                String CodePath = Assembly.GetExecutingAssembly().GetName().CodeBase;
                CodePath = CodePath.Substring(0, CodePath.LastIndexOf(@"\"));
                ruta = CodePath + @"\" + @"ayuda\ayuda.htm";

                Help.ShowHelp(this, ruta);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        /// <sumary> 
        ///  Inicializa el rectngulo y la imagen de fondo al cargar el form
        /// </sumary> 
        private void cargarFondo(object sender, EventArgs e)
        {
            imagenFondo = partida.cargarImagen(@"images\tablero1.jpg");
            rectFondo = new Rectangle(0, 0, imagenFondo.Width, imagenFondo.Height);
        }

        /// <sumary> 
        ///  Sobrecarga del mtodo OnPaintBackground que dibuja la imagen de fondo
        /// </sumary> 
        protected override void OnPaintBackground(PaintEventArgs e)
        {
            //base.OnPaintBackground(e);
            Graphics g = e.Graphics;
            g.DrawImage(imagenFondo, this.ClientRectangle, rectFondo, GraphicsUnit.Pixel);
        }


        #region Cdigo generado por el Diseador de Windows Forms

        /// <summary>
        /// Mtodo necesario para admitir el Diseador. No se puede modificar
        /// el contenido del mtodo con el editor de cdigo.
        /// </summary>
        private void InitializeComponent()
        {
            this.MenuPrincipal = new System.Windows.Forms.MainMenu();
            this.menuItem1 = new System.Windows.Forms.MenuItem();
            this.menuItem2 = new System.Windows.Forms.MenuItem();
            this.menuItem3 = new System.Windows.Forms.MenuItem();
            this.menuItem4 = new System.Windows.Forms.MenuItem();
            this.menuItem7 = new System.Windows.Forms.MenuItem();
            this.menuItem6 = new System.Windows.Forms.MenuItem();
            this.menuItem5 = new System.Windows.Forms.MenuItem();
            this.menuItem8 = new System.Windows.Forms.MenuItem();
            this.menuItem9 = new System.Windows.Forms.MenuItem();
            this.menuItem10 = new System.Windows.Forms.MenuItem();
            this.menuItem11 = new System.Windows.Forms.MenuItem();
            this.menuItem12 = new System.Windows.Forms.MenuItem();
            this.pictureBox1 = new System.Windows.Forms.PictureBox();
            this.menuItem13 = new System.Windows.Forms.MenuItem();
            this.SuspendLayout();
            // 
            // MenuPrincipal
            // 
            this.MenuPrincipal.MenuItems.Add(this.menuItem1);
            this.MenuPrincipal.MenuItems.Add(this.menuItem4);
            this.MenuPrincipal.MenuItems.Add(this.menuItem12);
            // 
            // menuItem1
            // 
            this.menuItem1.MenuItems.Add(this.menuItem2);
            this.menuItem1.MenuItems.Add(this.menuItem3);
            this.menuItem1.Text = "Juego";
            // 
            // menuItem2
            // 
            this.menuItem2.Text = "Nueva Partida";
            this.menuItem2.Click += new System.EventHandler(this.nuevaPartida);
            // 
            // menuItem3
            // 
            this.menuItem3.Text = "Salir";
            this.menuItem3.Click += new System.EventHandler(this.salir);
            // 
            // menuItem4
            // 
            this.menuItem4.MenuItems.Add(this.menuItem7);
            this.menuItem4.MenuItems.Add(this.menuItem8);
            this.menuItem4.Text = "Opciones";
            // 
            // menuItem7
            // 
            this.menuItem7.MenuItems.Add(this.menuItem6);
            this.menuItem7.MenuItems.Add(this.menuItem5);
            this.menuItem7.Text = "N jugadores";
            // 
            // menuItem6
            // 
            this.menuItem6.Checked = true;
            this.menuItem6.Text = "Jugador VS CPU";
            this.menuItem6.Click += new System.EventHandler(this.unJugador);
            // 
            // menuItem5
            // 
            this.menuItem5.Text = "2 Jugadores";
            this.menuItem5.Click += new System.EventHandler(this.dosJugadores);
            // 
            // menuItem8
            // 
            this.menuItem8.MenuItems.Add(this.menuItem9);
            this.menuItem8.MenuItems.Add(this.menuItem10);
            this.menuItem8.MenuItems.Add(this.menuItem11);
            this.menuItem8.Text = "Dificultad";
            // 
            // menuItem9
            // 
            this.menuItem9.Checked = true;
            this.menuItem9.Text = "Baja";
            this.menuItem9.Click += new System.EventHandler(this.dificultadBaja);
            // 
            // menuItem10
            // 
            this.menuItem10.Text = "Media";
            this.menuItem10.Click += new System.EventHandler(this.dificultadMedia);
            // 
            // menuItem11
            // 
            this.menuItem11.Text = "Alta";
            this.menuItem11.Click += new System.EventHandler(this.dificultadAlta);
            // 
            // menuItem12
            // 
            this.menuItem12.MenuItems.Add(this.menuItem13);
            this.menuItem12.Text = "Ayuda";
            // 
            // pictureBox1
            // 
            this.pictureBox1.Location = new System.Drawing.Point(109, 233);
            this.pictureBox1.Name = "pictureBox1";
            this.pictureBox1.Size = new System.Drawing.Size(25, 25);
            // 
            // menuItem13
            // 
            this.menuItem13.Text = "Ayuda";
            this.menuItem13.Click += new System.EventHandler(this.menuAyuda);
            // 
            // DamasForm
            // 
            this.AutoScaleDimensions = new System.Drawing.SizeF(96F, 96F);
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Dpi;
            this.AutoScroll = true;
            this.ClientSize = new System.Drawing.Size(240, 268);
            this.Controls.Add(this.pictureBox1);
            this.KeyPreview = true;
            this.Menu = this.MenuPrincipal;
            this.Name = "DamasForm";
            this.Text = "Pocket Damas";
            this.Load += new System.EventHandler(this.cargarFondo);
            this.ResumeLayout(false);

        }

        #endregion


        // Nueva partida
        private void nuevaPartida(object sender, EventArgs e)
        {
            // Guardamos el nivel y reiniciamos
            int level = partida.dificultad;
            partida.nuevoJuego();
            partida.dificultad = level;
        }

        // Salir de la aplicacin
        private void salir(object sender, EventArgs e)
        {
            Application.Exit();
        }

        // Dos jugadores
        private void dosJugadores(object sender, EventArgs e)
        {
            if (partida.juegaCPU)
            {
                partida.juegaCPU = false;
                menuItem5.Checked = true;
                menuItem6.Checked = false;
                int level = partida.dificultad;
                partida.nuevoJuego();
                partida.dificultad = level;
            }
        }

        // Jugador vs CPU
        private void unJugador(object sender, EventArgs e)
        {
            if (!partida.juegaCPU)
            {
                partida.juegaCPU = true;
                menuItem6.Checked = true;
                menuItem5.Checked = false;
                int level = partida.dificultad;
                partida.nuevoJuego();
                partida.dificultad = level;
            }
        }

        // Dificultad baja
        private void dificultadBaja(object sender, EventArgs e)
        {
            cambiaDificultad(2);
        }

        // Dificultad media
        private void dificultadMedia(object sender, EventArgs e)
        {
            cambiaDificultad(4);
        }

        // Dificultad alta
        private void dificultadAlta(object sender, EventArgs e)
        {
            cambiaDificultad(5);
        }

        // Ayuda
        private void menuAyuda(object sender, EventArgs e)
        {
            cargarAyuda();
        }

        /// <sumary> 
        /// Acerca de
        /// </sumary>
        /*private void acercaDe(object sender, EventArgs ev)
        {
            AboutBox acerca = new AboutBox();
            acerca.ShowDialog();
            acerca = null; // Help the GC
        }*/

    }
}
    