using System;
using System.Collections.Generic;
using System.Text;

namespace Proyecto0
{
  using System;
  using System.IO;
  using System.Windows.Forms;
  using System.Drawing;
  using System.Collections;
  using System.Threading;
  using System.Diagnostics;
  using System.Reflection;
  using Proyecto0;

  /// <sumary>
  ///   Clase control de la partida que maneja el tablero de juego.
  ///   Puede ser aadido a cualquier pantalla principal
  /// </sumary>    
  public class Partida : Control {

  // Imgenes utilizadas en la partida
  private Image fichaBlanca;
  private Image fichaNegra;
  private Image reinaBlanca;
  private Image reinaNegra;
  private Image cuadroBlanco;
  private Image cuadroNegro;
  private Image celdaSeleccionada;
  private Image fNegraSeleccionada;
  private Image fBlancaSeleccionada;
  private Image rNegraSeleccionada;
  private Image rBlancaSeleccionada;
    
  // Mantiene la informacin sobre el estado actual del tablero
  private Tablero tablero;

  // Coordenadas sobre la esquina superior izquierda del tablero
  private int coordX;
  private int coordY;

  // Tamao de las celdas
  private int tamCeldas;

  // Lista de celdas seleccionadas para una jugada
  // La primera posicin indica la primera celda en la que se inicia el movimiento.
  private Lista listaMov;

  // Jugador contrario manejado por la CPU
  private CPU cpu;
 
  // Juega la CPU?
  private Boolean CPUactiva = true;

  // Indica el tipo de movimiento invlido realizado para mostrarlo al jugador
  private static int tipoMovimientoInvalido = 0;
  
  // Formulario padre que tiene el control
  private DamasForm padre;

  // Pila temporal de tableros para almacenar movimientos mltiples
  private Stack tableros;


  /// <sumary>
  ///  Constructor principal de la clase que inicializa la partida
  /// </sumary>
  /// <param name="formPadre">
  ///  Formulario DamasForm padre en el que se agrega la partida
  /// </param>
  
  public Partida (DamasForm formPadre) {  
    listaMov = new Lista();
    tablero = new Tablero();
    padre = formPadre;
    CargarFichas();
    cpu = new CPU(tablero);
    reset();
    cambiarTurno();
  }
  

  /// <sumary>
  ///  Carga las imgenes de las fichas y celdas
  /// </sumary>
  public void CargarFichas()
  {
      String rutaNegra = @"images\Froja.bmp";
      String rutaBlanca = @"images\Fblanca.bmp";
      String rutaRNegra = @"images\ReinaRoja.bmp";
      String rutaRBlanca = @"images\ReinaBlanca.bmp";
      String rutaCNegro = @"images\CeldaOscura.bmp";
      String rutaCBlanco = @"images\CeldaClara.bmp";
      String rutaCSelecc = @"images\CeldaSeleccionada.bmp";
      String rutaNegraSelecc = @"images\FrojaSeleccionada.bmp";
      String rutaBlancaSelecc = @"images\FblancaSeleccionada.bmp";
      String rutaRNegraSelecc = @"images\RrojaSeleccionada.bmp";
      String rutaRBlancaSelecc = @"images\RblancaSeleccionada.bmp";

      fichaNegra = cargarImagen(rutaNegra);
      fichaBlanca = cargarImagen(rutaBlanca);
      reinaNegra = cargarImagen(rutaRNegra);
      reinaBlanca = cargarImagen(rutaRBlanca);
      cuadroNegro = cargarImagen(rutaCNegro);
      cuadroBlanco = cargarImagen(rutaCBlanco);
      celdaSeleccionada = cargarImagen(rutaCSelecc);
      fNegraSeleccionada = cargarImagen(rutaNegraSelecc);
      fBlancaSeleccionada = cargarImagen(rutaBlancaSelecc);
      rNegraSeleccionada = cargarImagen(rutaRNegraSelecc);
      rBlancaSeleccionada = cargarImagen(rutaRBlancaSelecc);


  }

  /// <sumary>
  ///  Carga una imagen de archivo y la devuelve como Image
  /// </sumary>
  public Image cargarImagen(String rutaImagen)
  {
      try
      {
          String CodePath =
              Assembly.GetExecutingAssembly().GetName().CodeBase;

          CodePath = CodePath.Substring(0, CodePath.LastIndexOf(@"\"));

          rutaImagen = CodePath + @"\" + rutaImagen;

          return new Bitmap(rutaImagen);
      }
      catch (System.IO.IOException)
      {
          return null;
      }
  }

  /// <sumary> 
  ///   Dificultad del juego (profundidad del minimax)
  /// </sumary>
  public int dificultad  {
    get {
      return cpu.profundidad;
    }
    set {
      cpu.profundidad = value;
    }
  }

  /// <sumary> 
  ///  Permite elegir la opcin de 2 jugadores o contra la CPU
  /// </sumary>
  public Boolean juegaCPU {
      get {
          return CPUactiva;
      }
      set {
          CPUactiva = value;
      }
  }

  /// <sumary> 
  ///  Indica el tipo de movimiento invlido que hemos realizado
  /// </sumary>
  public static int tipoMovInvalido
  {
      get
      {
          return tipoMovimientoInvalido;
      }
      set
      {
          tipoMovimientoInvalido = value;
      }
  }

  /// <sumary>
  ///  Empieza una nueva partida
  /// </sumary>
  public void nuevoJuego () {
    tablero.inicializarTablero ();
    listaMov.borrar ();
    Invalidate ();
    reset ();
    cambiarTurno ();
  }


  /// <sumary>
  ///  Cambia el turno que se muestra en el formulario padre
  /// </sumary>
  public void cambiarTurno() {
      if (tablero.getJugadorConTurno() == Tablero.BLANCA)
          padre.cambiaTurno(fichaBlanca);
      else
          padre.cambiaTurno(fichaNegra);
  }


  /// <sumary>
  ///  Sobrecarga del mtodo onPaint:
  ///  Dibuja tablero y fichas en respuesta al evento onPaint
  /// </sumary>
  /// <param name="ev">
  ///  Event arguments
  /// </param>
   protected override void OnPaint (PaintEventArgs ev) {
    Graphics g = ev.Graphics;
    Size d = ClientSize;
    int margenX;
    int margenY;
    int medidaCelda;
    

    // Calcula el incremento para pintar el tablero bien ajustado
    if (d.Width < d.Height) {
      margenX = 0;
      margenY = (d.Height - d.Width) / 2;
      
      medidaCelda = d.Width / 8;
    }
    else  {
      margenX = (d.Width - d.Height) / 2;
      margenY = 0;
      
      medidaCelda = d.Height / 8;
    }

    coordX = margenX;
    coordY = margenY;
    tamCeldas = medidaCelda;
    

    dibujaTablero(g, margenX, margenY, medidaCelda);
    dibujaFichas (g, margenX, margenY, medidaCelda);
 }

  /// <sumary>
  ///  Dibuja el tablero 
  /// </sumary>
  /// <param name="g">
  ///  Objeto Graphics usado para dibujar
  /// </param>
  /// <param name="margenX">
  ///  Margen horizontal del tablero
  /// </param>
  /// <param name="margenY">
  ///  Margen vertical del tablero
  /// </param>
  /// <param name="medidaCelda">
  ///  Medida de las casillas
  /// </param>
  private void dibujaTablero (Graphics g, int margenX, int margenY, int medidaCelda) {
    int pos;
    
    for (int y = 0; y < 8; y++)
      for (int x = 0; x < 8; x++) {
          if ((x + y) % 2 == 0)
          {
              // Celdas claras
              g.DrawImage(cuadroBlanco, margenX + x * medidaCelda, margenY + y * medidaCelda);
          }
          else
          {
              pos = y * 4 + (x + ((y % 2 == 0) ? -1 : 0)) / 2;

              if (listaMov.contiene(pos))
              {
                  // Celdas seleccionadas                  
                  if (listaMov.longitud() > 1)
                  {
                      //celdaSeleccionada vaca seleccionada
                      g.DrawImage(celdaSeleccionada, margenX + x * medidaCelda, margenY + y * medidaCelda);
                  }
                  else
                  {
                      if (tablero.getJugadorConTurno() == 2) 
                      {
                          if (tablero.getFicha(pos) == Tablero.REINA_BLANCA)
                              // Reina blanca seleccionada
                              g.DrawImage(rBlancaSeleccionada, margenX + x * medidaCelda, margenY + y * medidaCelda);
                          else
                              // Ficha blanca seleccionada
                              g.DrawImage(fBlancaSeleccionada, margenX + x * medidaCelda, margenY + y * medidaCelda);
                      }
                      else if (tablero.getJugadorConTurno() == 4)
                      {
                          if (tablero.getFicha(pos) == Tablero.REINA_NEGRA)
                              // Reina negra seleccionada
                              g.DrawImage(rNegraSeleccionada, margenX + x * medidaCelda, margenY + y * medidaCelda);
                          else
                              // Ficha negra seleccionada
                              g.DrawImage(fNegraSeleccionada, margenX + x * medidaCelda, margenY + y * medidaCelda);
                      }
                      else
                          g.DrawImage(celdaSeleccionada, margenX + x * medidaCelda, margenY + y * medidaCelda);
                  }
              }
              else
                  // Celdas oscuras sin seleccionar
                  g.DrawImage(cuadroNegro, margenX + x * medidaCelda, margenY + y * medidaCelda);
          }
           
      }
  }


  /// <sumary>
  ///  Dibuja las fichas
  /// </sumary>
  /// <param name="g">
  ///  Objeto Graphics usado para dibujar
  /// </param>
  /// <param name="margenX">
  ///  Margen horizontal del tablero
  /// </param>
  /// <param name="margenY">
  ///  Margen vertical del tablero
  /// </param>
  /// <param name="medidaCelda">
  ///  Medida de las casillas
  /// </param>
  private void dibujaFichas (Graphics g, int margenX, int margenY, int medidaCelda) {
    int x, y;

    for (int i = 0; i < 32; i++)
      try { 
          if (!listaMov.contiene(i))
          {
              if (tablero.getFicha(i) != Tablero.VACIA)
              {
                  y = i / 4;
                  x = (i % 4) * 2 + (y % 2 == 0 ? 1 : 0);
                  if (tablero.getFicha(i) == Tablero.NEGRA ||
                      tablero.getFicha(i) == Tablero.REINA_NEGRA)
                      g.DrawImage(fichaNegra, margenX + x * medidaCelda, margenY + y * medidaCelda);
                  
                  else
                      g.DrawImage(fichaBlanca, margenX + x * medidaCelda, margenY + y * medidaCelda);

                  if (tablero.getFicha(i) == Tablero.REINA_BLANCA)
                  {
                      g.DrawImage(reinaBlanca, margenX + x * medidaCelda, margenY + y * medidaCelda);
                  }
                  else if (tablero.getFicha(i) == Tablero.REINA_NEGRA)
                  {
                      g.DrawImage(reinaNegra, margenX + x * medidaCelda, margenY + y * medidaCelda);
                  }
              }
          }
      }
      catch (CoordErronea bad) {
        Debug.WriteLine (bad.StackTrace);
        Application.Exit ();
      }
  }
  

  /// <sumary>
  ///  Sobrecarga del mtodo OnMouseDown
  /// </sumary>
   protected override void OnMouseDown(MouseEventArgs e) {
    int pos;
    // En comentarios los tipos de movimientos que se pueden dar para que cambie el estado
    pos = posicionFicha (e.X , e.Y);
    if (pos != -1)
      try {
        int ficha = tablero.getFicha (pos);
        
        if (ficha != Tablero.VACIA &&
            (((ficha == Tablero.BLANCA || ficha == Tablero.REINA_BLANCA) &&
              tablero.getJugadorConTurno () == Tablero.BLANCA) ||
              ((ficha == Tablero.NEGRA || ficha == Tablero.REINA_NEGRA) &&
              tablero.getJugadorConTurno () == Tablero.NEGRA))) {
          if (listaMov.estaVacia ())
            listaMov.insertarCola (pos); // 1: Selecciono una ficha mia
          else {
            int temp = (int) listaMov.peekCola ();

            if (temp == pos) // 2: Pulso sobre ficha mia ya seleccionada y la deselecciono
              listaMov.extraerCola ();
            else {
              // 3: Tengo una ficha mia seleccionada y pulso otra ficha mia: deselecciono una y selecciono otra
              listaMov.extraerCola();
              listaMov.insertarCola(pos);
            }
          }


          Invalidate ();
          //Parent.Update();
          return;
        }
        else {
          bool correcto = false;
          Tablero tableroTemp;

          if (!listaMov.estaVacia())
          {
              // Copio el tablero actual en el temporal
              if (tableros.Count == 0)
              {
                  tableroTemp = (Tablero)tablero.clone();
                  tableros.Push(tableroTemp);
              }
              else
                  tableroTemp = (Tablero)tableros.Peek();


              int from = (int)listaMov.peekCola();
              if (tableroTemp.esMovimientoValido(from, pos))
              {
                  tableroTemp = (Tablero)tableroTemp.clone();

                  bool estaAtacando = tableroTemp.debeAtacar();

                  tableroTemp.mover(from, pos);

                  if (estaAtacando && tableroTemp.puedeAtacar(pos))
                  {
                      // 4: Aqu se ir pintando cada vez que se coma una ficha con posibilidad de comer otra ms (casilla marcada)
                      listaMov.insertarCola(pos);
                      tableros.Push(tableroTemp);

                  }
                  else
                  {
                      listaMov.insertarCola(pos);
                      // Aqu se pueden dar todos los demas movimientos: Comer, no comer, comer y seguir comiendo, hacer dama
                      // Ver ejecutaMovimientos
                      ejecutaMovimientos(listaMov, tablero);
                      tableros = new Stack();
                  }

                  correcto = true;
              }
              else if (from == pos)
              {
                  listaMov.extraerCola();
                  tableros.Pop();

                  correcto = true;
              }
          }
          else
            // Debe seleccionar ficha del jugador que tenga el turno
            tipoMovInvalido = 1;
          
          
          if (!correcto) {
            // Movimientos invlidos
            if (tipoMovInvalido == 1) 
            {
                MessageBox.Show("Turno para las fichas " + tablero.getColorTurno(), "Seleccin incorrecta");
                tipoMovInvalido = 0;
            }
            else if (tipoMovInvalido == 2)
            {
                MessageBox.Show("Esa casilla est ocupada", "Movimiento invlido");
                tipoMovInvalido = 0;
            }
            else if (tipoMovInvalido == 3)
            {
                MessageBox.Show("Debes avanzar en diagonal\n sin retroceder", "Movimiento invlido");
                tipoMovInvalido = 0;
            }
            else if (tipoMovInvalido == 4)
            {
                MessageBox.Show("Debes comer ficha", "Movimiento invlido");
                tipoMovInvalido = 0;
            }
            else
            {
                MessageBox.Show("Movimiento invlido", "Error");
                tipoMovInvalido = 0;
            }
                
          }
          else {
            Invalidate ();
            //Parent.Update();
          }
        }
      }
      catch (CoordErronea bad) {
        Debug.WriteLine (bad.StackTrace);
        Application.Exit ();
      }
      catch (MovInvalido bad) {
        Debug.WriteLine (bad.StackTrace);
        Application.Exit ();
      }
    }


    /// <sumary>
    ///  Borra e inicializa la variable tableros
    /// </sumary>
    public void reset () {
      tableros = new Stack ();
    } 
  

  /// <sumary>
  ///  Aplica los movimientos sobre el tablero y si est jugando la CPU le pasa el turno para que juegue
  /// </sumary>
  /// <param name="movs">
  ///  Lista de movimientos
  /// </param>
  /// <param name="tablero">
  ///  El tablero al que se le aplicarn los movimientos
  /// </param>
  private void ejecutaMovimientos (Lista movs, Tablero tablero) {
    Lista movimientos = new Lista ();
    int from, to = 0;

    from = (int) movs.extraerCabeza ();
    while (!movs.estaVacia ()) {
      to = (int) movs.extraerCabeza ();
      movimientos.insertarCola (new Movimiento (from, to));
      from = to;
    }
    
    // Aqu se llamar a la funcin para mover 
    tablero.mover (movimientos);
    // Aqu se llamar a la funcin para mover
    Invalidate ();
    //Parent.Update();
    listaMov.borrar ();
    reset ();
        

    if (!juegoTerminado ()) {
      
      
      
      // Si estamos jugando contra la CPU cambiamos el turno al atributo cpu para que mueva
      if (juegaCPU)
      {
          cambiarTurno();
          Thread.Sleep(1000); // Tiempo para ver el movimiento
          cpu.play();
          Invalidate();
          //Parent.Update();
      }

      if (!juegoTerminado ())
        cambiarTurno ();
    }
  }
    
  /// <sumary>
  ///  Devuelve la posicin de una ficha
  /// </sumary>
  /// <param name="posX">
  ///  Coordenada X
  /// </param>
  /// <param name="posY">
  ///  Coordenada Y
  /// </param>
  /// <value>
  ///  Posicin de la ficha: 0..31. Si no existe devolvemos -1
  /// </value>
  private int posicionFicha (int posX, int posY) {
    for (int i = 0; i < 32; i++) {
      int x, y;

      y = i / 4;
      x = (i % 4) * 2 + (y % 2 == 0 ? 1 : 0);
      if (coordX + x * tamCeldas < posX &&
          posX < coordX + (x + 1) * tamCeldas &&
          coordY + y * tamCeldas < posY &&
          posY < coordY + (y + 1) * tamCeldas)
        return i;
    }

    return -1;
  }

  /// <sumary>
  ///  Comprueba si la partida ha terminado
  /// </sumary>
  /// <value>
  ///  Devuelve true si la partida ha terminado y false en caso contrario
  /// </value>
  private bool juegoTerminado () {
    bool result;

    int white = tablero.getFichasBlancas ();
    int black = tablero.getFichasNegras ();
    if (tablero.juegoTerminado ()) {
      if (tablero.ganador () == Tablero.NEGRA)
        MessageBox.Show ("Las rojas ganan", "Juego terminado");
      else
        MessageBox.Show ("Ganan las blancas", "Juego terminado");
      result = true;
    }
    else
      result = false;

    return result;
  }
 }
}
